@@ -23,14 +23,14 @@ int FAST_FUNC xgetpty(char *line)
 			const char *name;
 			name = ptsname(p); /* find out the name of slave pty */
 			if (!name) {
-				bb_perror_msg_and_die("ptsname error (is /dev/pts mounted?)");
+				bb_simple_perror_msg_and_die("ptsname error (is /dev/pts mounted?)");
 			}
 			safe_strncpy(line, name, GETPTY_BUFSIZE);
 		}
 # else
 		/* find out the name of slave pty */
 		if (ptsname_r(p, line, GETPTY_BUFSIZE-1) != 0) {
-			bb_perror_msg_and_die("ptsname error (is /dev/pts mounted?)");
+			bb_simple_perror_msg_and_die("ptsname error (is /dev/pts mounted?)");
 		}
 		line[GETPTY_BUFSIZE-1] = '\0';
 # endif
@@ -61,5 +61,5 @@ int FAST_FUNC xgetpty(char *line)
 		}
 	}
 #endif /* FEATURE_DEVPTS */
-	bb_error_msg_and_die("can't find free pty");
+	bb_simple_error_msg_and_die("can't find free pty");
 }
