@@ -8,25 +8,25 @@
  * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 //config:config MODPROBE
-//config:	bool "modprobe"
+//config:	bool "modprobe (28 kb)"
 //config:	default y
 //config:	select PLATFORM_LINUX
 //config:	help
-//config:	  Handle the loading of modules, and their dependencies on a high
-//config:	  level.
+//config:	Handle the loading of modules, and their dependencies on a high
+//config:	level.
 //config:
 //config:config FEATURE_MODPROBE_BLACKLIST
 //config:	bool "Blacklist support"
 //config:	default y
 //config:	depends on MODPROBE && !MODPROBE_SMALL
 //config:	help
-//config:	  Say 'y' here to enable support for the 'blacklist' command in
-//config:	  modprobe.conf. This prevents the alias resolver to resolve
-//config:	  blacklisted modules. This is useful if you want to prevent your
-//config:	  hardware autodetection scripts to load modules like evdev, frame
-//config:	  buffer drivers etc.
+//config:	Say 'y' here to enable support for the 'blacklist' command in
+//config:	modprobe.conf. This prevents the alias resolver to resolve
+//config:	blacklisted modules. This is useful if you want to prevent your
+//config:	hardware autodetection scripts to load modules like evdev, frame
+//config:	buffer drivers etc.
 
-//applet:IF_MODPROBE(IF_NOT_MODPROBE_SMALL(APPLET(modprobe, BB_DIR_SBIN, BB_SUID_DROP)))
+//applet:IF_MODPROBE(IF_NOT_MODPROBE_SMALL(APPLET_NOEXEC(modprobe, modprobe, BB_DIR_SBIN, BB_SUID_DROP, modprobe)))
 
 //kbuild:ifneq ($(CONFIG_MODPROBE_SMALL),y)
 //kbuild:lib-$(CONFIG_MODPROBE) += modprobe.o modutils.o
@@ -133,7 +133,7 @@
  */
 #define MODPROBE_OPTS  "alrDb"
 /* -a and -D _are_ in fact compatible */
-#define MODPROBE_COMPLEMENTARY ("q-v:v-q:l--arD:r--alD:a--lr:D--rl")
+#define MODPROBE_COMPLEMENTARY "q-v:v-q:l--arD:r--alD:a--lr:D--rl"
 //#define MODPROBE_OPTS  "acd:lnrt:C:b"
 //#define MODPROBE_COMPLEMENTARY "q-v:v-q:l--acr:a--lr:r--al"
 enum {
@@ -245,7 +245,7 @@ static int FAST_FUNC config_file_action(const char *filename,
 	parser_t *p;
 	struct module_entry *m;
 	int rc = TRUE;
-	const char *base, *ext;
+	const char *base;
 
 	/* Skip files that begin with a "." */
 	base = bb_basename(filename);
@@ -266,8 +266,7 @@ static int FAST_FUNC config_file_action(const char *filename,
 	 * "include FILE_NOT_ENDING_IN_CONF" must work too.
 	 */
 	if (depth != 0) {
-		ext = strrchr(base, '.');
-		if (ext == NULL || strcmp(ext + 1, "conf"))
+		if (!is_suffixed_with(base, ".conf"))
 			goto error;
 	}
 
@@ -566,9 +565,10 @@ int modprobe_main(int argc UNUSED_PARAM, char **argv)
 
 	INIT_G();
 
-	IF_LONG_OPTS(applet_long_options = modprobe_longopts;)
-	opt_complementary = MODPROBE_COMPLEMENTARY;
-	opt = getopt32(argv, INSMOD_OPTS MODPROBE_OPTS INSMOD_ARGS);
+	opt = getopt32long(argv, "^" INSMOD_OPTS MODPROBE_OPTS "\0" MODPROBE_COMPLEMENTARY,
+			modprobe_longopts
+			INSMOD_ARGS
+	);
 	argv += optind;
 
 	/* Goto modules location */
