@@ -95,9 +95,7 @@
  * If -c is not set, an attempt will be made to open the default
  * root configuration file.  If -c is set and the file is not found, the
  * server exits with an error.
- *
  */
- /* TODO: use TCP_CORK, parse_config() */
 //config:config HTTPD
 //config:	bool "httpd (32 kb)"
 //config:	default y
@@ -246,6 +244,8 @@
 //usage:     "\n	-e STRING	HTML encode STRING"
 //usage:     "\n	-d STRING	URL decode STRING"
 
+/* TODO: use TCP_CORK, parse_config() */
+
 #include "libbb.h"
 #include "common_bufsiz.h"
 #if ENABLE_PAM
@@ -259,17 +259,11 @@
 #if ENABLE_FEATURE_USE_SENDFILE
 # include <sys/sendfile.h>
 #endif
-/* amount of buffering in a pipe */
-#ifndef PIPE_BUF
-# define PIPE_BUF 4096
-#endif
 
 #define DEBUG 0
 
 #define IOBUF_SIZE 8192
-#if PIPE_BUF >= IOBUF_SIZE
-# error "PIPE_BUF >= IOBUF_SIZE"
-#endif
+#define MAX_HTTP_HEADERS_SIZE (32*1024)
 
 #define HEADER_READ_TIMEOUT 60
 
@@ -305,6 +299,13 @@ typedef struct Htaccess_Proxy {
 	char *url_to;
 } Htaccess_Proxy;
 
+typedef enum CGI_type {
+	CGI_NONE = 0,
+	CGI_NORMAL,
+	CGI_INDEX,
+	CGI_INTERPRETER,
+} CGI_type;
+
 enum {
 	HTTP_OK = 200,
 	HTTP_PARTIAL_CONTENT = 206,
@@ -316,6 +317,7 @@ enum {
 	HTTP_REQUEST_TIMEOUT = 408,
 	HTTP_NOT_IMPLEMENTED = 501,   /* used for unrecognized requests */
 	HTTP_INTERNAL_SERVER_ERROR = 500,
+	HTTP_ENTITY_TOO_LARGE = 413,
 	HTTP_CONTINUE = 100,
 #if 0   /* future use */
 	HTTP_SWITCHING_PROTOCOLS = 101,
@@ -347,6 +349,7 @@ static const uint16_t http_response_type[] ALIGN2 = {
 	HTTP_BAD_REQUEST,
 	HTTP_FORBIDDEN,
 	HTTP_INTERNAL_SERVER_ERROR,
+	HTTP_ENTITY_TOO_LARGE,
 #if 0   /* not implemented */
 	HTTP_CREATED,
 	HTTP_ACCEPTED,
@@ -377,6 +380,7 @@ static const struct {
 	{ "Bad Request", "Unsupported method" },
 	{ "Forbidden", ""  },
 	{ "Internal Server Error", "Internal Server Error" },
+	{ "Entity Too Large", "Entity Too Large" },
 #if 0   /* not implemented */
 	{ "Created" },
 	{ "Accepted" },
@@ -396,12 +400,11 @@ struct globals {
 	/* client can handle gzip / we are going to send gzip */
 	smallint content_gzip;
 #endif
-	unsigned rmt_ip;        /* used for IP-based allow/deny rules */
 	time_t last_mod;
 	char *rmt_ip_str;       /* for $REMOTE_ADDR and $REMOTE_PORT */
 	const char *bind_addr_or_port;
 
-	const char *g_query;
+	char *g_query;
 	const char *opt_c_configFile;
 	const char *home_httpd;
 	const char *index_page;
@@ -412,11 +415,6 @@ struct globals {
 
 	IF_FEATURE_HTTPD_BASIC_AUTH(const char *g_realm;)
 	IF_FEATURE_HTTPD_BASIC_AUTH(char *remoteuser;)
-	IF_FEATURE_HTTPD_CGI(char *referer;)
-	IF_FEATURE_HTTPD_CGI(char *user_agent;)
-	IF_FEATURE_HTTPD_CGI(char *host;)
-	IF_FEATURE_HTTPD_CGI(char *http_accept;)
-	IF_FEATURE_HTTPD_CGI(char *http_accept_language;)
 
 	off_t file_size;        /* -1 - unknown */
 #if ENABLE_FEATURE_HTTPD_RANGES
@@ -452,7 +450,6 @@ struct globals {
 #else
 # define content_gzip     0
 #endif
-#define rmt_ip            (G.rmt_ip           )
 #define bind_addr_or_port (G.bind_addr_or_port)
 #define g_query           (G.g_query          )
 #define opt_c_configFile  (G.opt_c_configFile )
@@ -1028,7 +1025,7 @@ static void log_and_exit(void)
 	*/
 
 	if (verbose > 2)
-		bb_error_msg("closed");
+		bb_simple_error_msg("closed");
 	_exit(xfunc_error_retval);
 }
 
@@ -1146,18 +1143,61 @@ static void send_headers(unsigned responseNum)
 			file_size = range_end - range_start + 1;
 		}
 #endif
+
+//RFC 2616 4.4 Message Length
+// The transfer-length of a message is the length of the message-body as
+// it appears in the message; that is, after any transfer-codings have
+// been applied. When a message-body is included with a message, the
+// transfer-length of that body is determined by one of the following
+// (in order of precedence):
+// 1.Any response message which "MUST NOT" include a message-body (such
+//   as the 1xx, 204, and 304 responses and any response to a HEAD
+//   request) is always terminated by the first empty line after the
+//   header fields, regardless of the entity-header fields present in
+//   the message.
+// 2.If a Transfer-Encoding header field (section 14.41) is present and
+//   has any value other than "identity", then the transfer-length is
+//   defined by use of the "chunked" transfer-coding (section 3.6),
+//   unless the message is terminated by closing the connection.
+// 3.If a Content-Length header field (section 14.13) is present, its
+//   decimal value in OCTETs represents both the entity-length and the
+//   transfer-length. The Content-Length header field MUST NOT be sent
+//   if these two lengths are different (i.e., if a Transfer-Encoding
+//   header field is present). If a message is received with both a
+//   Transfer-Encoding header field and a Content-Length header field,
+//   the latter MUST be ignored.
+// 4.If the message uses the media type "multipart/byteranges" ...
+// 5.By the server closing the connection.
+//
+// (NB: standards do not define "Transfer-Length:" _header_,
+// transfer-length above is just a concept).
+
 		len += sprintf(iobuf + len,
 #if ENABLE_FEATURE_HTTPD_RANGES
 			"Accept-Ranges: bytes\r\n"
 #endif
 			"Last-Modified: %s\r\n"
-			"%s-Length: %"OFF_FMT"u\r\n",
+	/* Because of 4.4 (5), we can forgo sending of "Content-Length"
+	 * since we close connection afterwards, but it helps clients
+	 * to e.g. estimate download times, show progress bars etc.
+	 * Theoretically we should not send it if page is compressed,
+	 * but de-facto standard is to send it (see comment below).
+	 */
+			"Content-Length: %"OFF_FMT"u\r\n",
 				date_str,
-				content_gzip ? "Transfer" : "Content",
 				file_size
 		);
 	}
 
+	/* This should be "Transfer-Encoding", not "Content-Encoding":
+	 * "data is compressed for transfer", not "data is an archive".
+	 * But many clients were not handling "Transfer-Encoding" correctly
+	 * (they were not uncompressing gzipped pages, tried to show
+	 * raw compressed data), and servers worked around it by using
+	 * "Content-Encoding" instead... and this become de-facto standard.
+	 * https://bugzilla.mozilla.org/show_bug.cgi?id=68517
+	 * https://bugs.chromium.org/p/chromium/issues/detail?id=94730
+	 */
 	if (content_gzip)
 		len += sprintf(iobuf + len, "Content-Encoding: gzip\r\n");
 
@@ -1180,7 +1220,7 @@ static void send_headers(unsigned responseNum)
 	}
 	if (full_write(STDOUT_FILENO, iobuf, len) != len) {
 		if (verbose > 1)
-			bb_perror_msg("error");
+			bb_simple_perror_msg("error");
 		log_and_exit();
 	}
 }
@@ -1194,37 +1234,39 @@ static void send_headers_and_exit(int responseNum)
 }
 
 /*
- * Read from the socket until '\n' or EOF. '\r' chars are removed.
+ * Read from the socket until '\n' or EOF.
+ * '\r' chars are removed.
  * '\n' is replaced with NUL.
  * Return number of characters read or 0 if nothing is read
  * ('\r' and '\n' are not counted).
  * Data is returned in iobuf.
  */
-static int get_line(void)
+static unsigned get_line(void)
 {
-	int count = 0;
+	unsigned count;
 	char c;
 
-	alarm(HEADER_READ_TIMEOUT);
+	count = 0;
 	while (1) {
 		if (hdr_cnt <= 0) {
+			alarm(HEADER_READ_TIMEOUT);
 			hdr_cnt = safe_read(STDIN_FILENO, hdr_buf, sizeof_hdr_buf);
 			if (hdr_cnt <= 0)
-				break;
+				goto ret;
 			hdr_ptr = hdr_buf;
 		}
-		iobuf[count] = c = *hdr_ptr++;
 		hdr_cnt--;
-
+		c = *hdr_ptr++;
 		if (c == '\r')
 			continue;
-		if (c == '\n') {
-			iobuf[count] = '\0';
+		if (c == '\n')
 			break;
-		}
+		iobuf[count] = c;
 		if (count < (IOBUF_SIZE - 1))      /* check overflow */
 			count++;
 	}
+ ret:
+	iobuf[count] = '\0';
 	return count;
 }
 
@@ -1364,10 +1406,10 @@ static NOINLINE void cgi_io_loop_and_exit(int fromCgi_rd, int toCgi_wr, int post
 				 * CGI may output a few first bytes and then wait
 				 * for POSTDATA without closing stdout.
 				 * With full_read we may wait here forever. */
-				count = safe_read(fromCgi_rd, rbuf + out_cnt, PIPE_BUF - 8);
+				count = safe_read(fromCgi_rd, rbuf + out_cnt, IOBUF_SIZE - 8);
 				if (count <= 0) {
 					/* eof (or error) and there was no "HTTP",
-					 * so write it, then write received data */
+					 * send "HTTP/1.0 200 OK\r\n", then send received data */
 					if (out_cnt) {
 						full_write(STDOUT_FILENO, HTTP_200, sizeof(HTTP_200)-1);
 						full_write(STDOUT_FILENO, rbuf, out_cnt);
@@ -1405,7 +1447,7 @@ static NOINLINE void cgi_io_loop_and_exit(int fromCgi_rd, int toCgi_wr, int post
 					out_cnt = -1; /* buffering off */
 				}
 			} else {
-				count = safe_read(fromCgi_rd, rbuf, PIPE_BUF);
+				count = safe_read(fromCgi_rd, rbuf, IOBUF_SIZE);
 				if (count <= 0)
 					break;  /* eof (or error) */
 			}
@@ -1437,23 +1479,17 @@ static void setenv1(const char *name, const char *value)
  * const char *url              The requested URL (with leading /).
  * const char *orig_uri         The original URI before rewriting (if any)
  * int post_len                 Length of the POST body.
- * const char *cookie           For set HTTP_COOKIE.
- * const char *content_type     For set CONTENT_TYPE.
  */
 static void send_cgi_and_exit(
 		const char *url,
 		const char *orig_uri,
 		const char *request,
-		int post_len,
-		const char *cookie,
-		const char *content_type) NORETURN;
+		int post_len) NORETURN;
 static void send_cgi_and_exit(
 		const char *url,
 		const char *orig_uri,
 		const char *request,
-		int post_len,
-		const char *cookie,
-		const char *content_type)
+		int post_len)
 {
 	struct fd_pair fromCgi;  /* CGI -> httpd pipe */
 	struct fd_pair toCgi;    /* httpd -> CGI pipe */
@@ -1531,26 +1567,14 @@ static void send_cgi_and_exit(
 #endif
 		}
 	}
-	setenv1("HTTP_USER_AGENT", G.user_agent);
-	if (G.http_accept)
-		setenv1("HTTP_ACCEPT", G.http_accept);
-	if (G.http_accept_language)
-		setenv1("HTTP_ACCEPT_LANGUAGE", G.http_accept_language);
 	if (post_len)
 		putenv(xasprintf("CONTENT_LENGTH=%u", post_len));
-	if (cookie)
-		setenv1("HTTP_COOKIE", cookie);
-	if (content_type)
-		setenv1("CONTENT_TYPE", content_type);
 #if ENABLE_FEATURE_HTTPD_BASIC_AUTH
 	if (remoteuser) {
 		setenv1("REMOTE_USER", remoteuser);
 		putenv((char*)"AUTH_TYPE=Basic");
 	}
 #endif
-	if (G.referer)
-		setenv1("HTTP_REFERER", G.referer);
-	setenv1("HTTP_HOST", G.host); /* set to "" if NULL */
 	/* setenv1("SERVER_NAME", safe_gethostname()); - don't do this,
 	 * just run "env SERVER_NAME=xyz httpd ..." instead */
 
@@ -1702,8 +1726,10 @@ static NOINLINE void send_file_and_exit(const char *url, int what)
 			".jpg.jpeg\0" "image/jpeg\0"
 			".gif\0"      "image/gif\0"
 			".png\0"      "image/png\0"
+			".svg\0"      "image/svg+xml\0"
 			/* .css line must be after .c line */
 			".css\0"      "text/css\0"
+			".js\0"       "application/javascript\0"
 			".wav\0"      "audio/wav\0"
 			".avi\0"      "video/x-msvideo\0"
 			".qt.mov\0"   "video/quicktime\0"
@@ -1812,12 +1838,12 @@ static NOINLINE void send_file_and_exit(const char *url, int what)
 	if (count < 0) {
  IF_FEATURE_USE_SENDFILE(fin:)
 		if (verbose > 1)
-			bb_perror_msg("error");
+			bb_simple_perror_msg("error");
 	}
 	log_and_exit();
 }
 
-static int checkPermIP(void)
+static void if_ip_denied_send_HTTP_FORBIDDEN_and_exit(unsigned remote_ip)
 {
 	Htaccess_IP *cur;
 
@@ -1836,11 +1862,15 @@ static int checkPermIP(void)
 			(unsigned char)(cur->mask)
 		);
 #endif
-		if ((rmt_ip & cur->mask) == cur->ip)
-			return (cur->allow_deny == 'A'); /* A -> 1 */
+		if ((remote_ip & cur->mask) == cur->ip) {
+			if (cur->allow_deny == 'A')
+				return;
+			send_headers_and_exit(HTTP_FORBIDDEN);
+		}
 	}
 
-	return !flg_deny_all; /* depends on whether we saw "D:*" */
+	if (flg_deny_all) /* depends on whether we saw "D:*" */
+		send_headers_and_exit(HTTP_FORBIDDEN);
 }
 
 #if ENABLE_FEATURE_HTTPD_BASIC_AUTH
@@ -2077,12 +2107,15 @@ static void handle_incoming_and_exit(const len_and_sockaddr *fromAddr)
 	char *urlcopy;
 	char *urlp;
 	char *tptr;
+	unsigned remote_ip;
+#if ENABLE_FEATURE_HTTPD_CGI
+	unsigned total_headers_len;
+#endif
 #if ENABLE_FEATURE_HTTPD_CGI
 	static const char request_HEAD[] ALIGN1 = "HEAD";
 	const char *prequest;
-	char *cookie = NULL;
-	char *content_type = NULL;
 	unsigned long length = 0;
+	enum CGI_type cgi_type = CGI_NONE;
 #elif ENABLE_FEATURE_HTTPD_PROXY
 #define prequest request_GET
 	unsigned long length = 0;
@@ -2090,29 +2123,22 @@ static void handle_incoming_and_exit(const len_and_sockaddr *fromAddr)
 #if ENABLE_FEATURE_HTTPD_BASIC_AUTH
 	smallint authorized = -1;
 #endif
-	smallint ip_allowed;
-	char http_major_version;
-#if ENABLE_FEATURE_HTTPD_PROXY
-	char http_minor_version;
-	char *header_buf = header_buf; /* for gcc */
-	char *header_ptr = header_ptr;
-	Htaccess_Proxy *proxy_entry;
-#endif
+	char *HTTP_slash;
 
 	/* Allocation of iobuf is postponed until now
 	 * (IOW, server process doesn't need to waste 8k) */
 	iobuf = xmalloc(IOBUF_SIZE);
 
-	rmt_ip = 0;
+	remote_ip = 0;
 	if (fromAddr->u.sa.sa_family == AF_INET) {
-		rmt_ip = ntohl(fromAddr->u.sin.sin_addr.s_addr);
+		remote_ip = ntohl(fromAddr->u.sin.sin_addr.s_addr);
 	}
 #if ENABLE_FEATURE_IPV6
 	if (fromAddr->u.sa.sa_family == AF_INET6
 	 && fromAddr->u.sin6.sin6_addr.s6_addr32[0] == 0
 	 && fromAddr->u.sin6.sin6_addr.s6_addr32[1] == 0
 	 && ntohl(fromAddr->u.sin6.sin6_addr.s6_addr32[2]) == 0xffff)
-		rmt_ip = ntohl(fromAddr->u.sin6.sin6_addr.s6_addr32[3]);
+		remote_ip = ntohl(fromAddr->u.sin6.sin6_addr.s6_addr32[3]);
 #endif
 	if (ENABLE_FEATURE_HTTPD_CGI || DEBUG || verbose) {
 		/* NB: can be NULL (user runs httpd -i by hand?) */
@@ -2123,8 +2149,9 @@ static void handle_incoming_and_exit(const len_and_sockaddr *fromAddr)
 		if (rmt_ip_str)
 			applet_name = rmt_ip_str;
 		if (verbose > 2)
-			bb_error_msg("connected");
+			bb_simple_error_msg("connected");
 	}
+	if_ip_denied_send_HTTP_FORBIDDEN_and_exit(remote_ip);
 
 	/* Install timeout handler. get_line() needs it. */
 	signal(SIGALRM, send_REQUEST_TIMEOUT_and_exit);
@@ -2158,31 +2185,59 @@ static void handle_incoming_and_exit(const len_and_sockaddr *fromAddr)
 	if (urlp[0] != '/')
 		send_headers_and_exit(HTTP_BAD_REQUEST);
 
-	/* Find end of URL and parse HTTP version, if any */
-	http_major_version = '0';
-	IF_FEATURE_HTTPD_PROXY(http_minor_version = '0';)
-	tptr = strchrnul(urlp, ' ');
+	/* Find end of URL */
+	HTTP_slash = strchr(urlp, ' ');
 	/* Is it " HTTP/"? */
-	if (tptr[0] && strncmp(tptr + 1, HTTP_200, 5) == 0) {
-		http_major_version = tptr[6];
-		IF_FEATURE_HTTPD_PROXY(http_minor_version = tptr[8];)
-	}
-	*tptr = '\0';
+	if (!HTTP_slash || strncmp(HTTP_slash + 1, HTTP_200, 5) != 0)
+		send_headers_and_exit(HTTP_BAD_REQUEST);
+	*HTTP_slash++ = '\0';
 
 	/* Copy URL from after "GET "/"POST " to stack-allocated char[] */
-	urlcopy = alloca((tptr - urlp) + 2 + strlen(index_page));
+	urlcopy = alloca((HTTP_slash - urlp) + 2 + strlen(index_page));
 	/*if (urlcopy == NULL)
 	 *	send_headers_and_exit(HTTP_INTERNAL_SERVER_ERROR);*/
 	strcpy(urlcopy, urlp);
 	/* NB: urlcopy ptr is never changed after this */
 
-	/* Extract url args if present */
-	/* g_query = NULL; - already is */
-	tptr = strchr(urlcopy, '?');
-	if (tptr) {
-		*tptr++ = '\0';
-		g_query = tptr;
+#if ENABLE_FEATURE_HTTPD_PROXY
+	{
+		int proxy_fd;
+		len_and_sockaddr *lsa;
+		Htaccess_Proxy *proxy_entry = find_proxy_entry(urlcopy);
+
+		if (proxy_entry) {
+			if (verbose > 1)
+				bb_error_msg("proxy:%s", urlcopy);
+			lsa = host2sockaddr(proxy_entry->host_port, 80);
+			if (!lsa)
+				send_headers_and_exit(HTTP_INTERNAL_SERVER_ERROR);
+			proxy_fd = socket(lsa->u.sa.sa_family, SOCK_STREAM, 0);
+			if (proxy_fd < 0)
+				send_headers_and_exit(HTTP_INTERNAL_SERVER_ERROR);
+			if (connect(proxy_fd, &lsa->u.sa, lsa->len) < 0)
+				send_headers_and_exit(HTTP_INTERNAL_SERVER_ERROR);
+			/* Disable peer header reading timeout */
+			alarm(0);
+			/* Config directive was of the form:
+			 *   P:/url:[http://]hostname[:port]/new/path
+			 * When /urlSFX is requested, reverse proxy it
+			 * to http://hostname[:port]/new/pathSFX
+			 */
+			fdprintf(proxy_fd, "%s %s%s %s\r\n",
+					prequest, /* "GET" or "POST" */
+					proxy_entry->url_to, /* "/new/path" */
+					urlcopy + strlen(proxy_entry->url_from), /* "SFX" */
+					HTTP_slash /* "HTTP/xyz" */
+			);
+			cgi_io_loop_and_exit(proxy_fd, proxy_fd, /*max POST length:*/ INT_MAX);
+		}
 	}
+#endif
+
+	/* Extract url args if present */
+	g_query = strchr(urlcopy, '?');
+	if (g_query)
+		*g_query++ = '\0';
 
 	/* Decode URL escape sequences */
 	tptr = percent_decode_in_place(urlcopy, /*strict:*/ 1);
@@ -2222,7 +2277,7 @@ static void handle_incoming_and_exit(const len_and_sockaddr *fromAddr)
 			}
 		}
 		*++urlp = *tptr;
-		if (*urlp == '\0')
+		if (*tptr == '\0')
 			break;
  next_char:
 		tptr++;
@@ -2240,193 +2295,17 @@ static void handle_incoming_and_exit(const len_and_sockaddr *fromAddr)
 		bb_error_msg("url:%s", urlcopy);
 
 	tptr = urlcopy;
-	ip_allowed = checkPermIP();
-	while (ip_allowed && (tptr = strchr(tptr + 1, '/')) != NULL) {
+	while ((tptr = strchr(tptr + 1, '/')) != NULL) {
 		/* have path1/path2 */
 		*tptr = '\0';
 		if (is_directory(urlcopy + 1, /*followlinks:*/ 1)) {
 			/* may have subdir config */
 			parse_conf(urlcopy + 1, SUBDIR_PARSE);
-			ip_allowed = checkPermIP();
+			if_ip_denied_send_HTTP_FORBIDDEN_and_exit(remote_ip);
 		}
 		*tptr = '/';
 	}
 
-#if ENABLE_FEATURE_HTTPD_PROXY
-	proxy_entry = find_proxy_entry(urlcopy);
-	if (proxy_entry)
-		header_buf = header_ptr = xmalloc(IOBUF_SIZE);
-#endif
-
-	if (http_major_version >= '0') {
-		/* Request was with "... HTTP/nXXX", and n >= 0 */
-
-		/* Read until blank line */
-		while (1) {
-			if (!get_line())
-				break; /* EOF or error or empty line */
-			if (DEBUG)
-				bb_error_msg("header: '%s'", iobuf);
-
-#if ENABLE_FEATURE_HTTPD_PROXY
-			/* We need 2 more bytes for yet another "\r\n" -
-			 * see near fdprintf(proxy_fd...) further below */
-			if (proxy_entry && (header_ptr - header_buf) < IOBUF_SIZE - 4) {
-				int len = strnlen(iobuf, IOBUF_SIZE - (header_ptr - header_buf) - 4);
-				memcpy(header_ptr, iobuf, len);
-				header_ptr += len;
-				header_ptr[0] = '\r';
-				header_ptr[1] = '\n';
-				header_ptr += 2;
-			}
-#endif
-
-#if ENABLE_FEATURE_HTTPD_CGI || ENABLE_FEATURE_HTTPD_PROXY
-			/* Try and do our best to parse more lines */
-			if ((STRNCASECMP(iobuf, "Content-Length:") == 0)) {
-				/* extra read only for POST */
-				if (prequest != request_GET
-# if ENABLE_FEATURE_HTTPD_CGI
-				 && prequest != request_HEAD
-# endif
-				) {
-					tptr = skip_whitespace(iobuf + sizeof("Content-Length:") - 1);
-					if (!tptr[0])
-						send_headers_and_exit(HTTP_BAD_REQUEST);
-					/* not using strtoul: it ignores leading minus! */
-					length = bb_strtou(tptr, NULL, 10);
-					/* length is "ulong", but we need to pass it to int later */
-					if (errno || length > INT_MAX)
-						send_headers_and_exit(HTTP_BAD_REQUEST);
-				}
-			}
-#endif
-#if ENABLE_FEATURE_HTTPD_CGI
-			else if (STRNCASECMP(iobuf, "Cookie:") == 0) {
-				if (!cookie) /* in case they send millions of these, do not OOM */
-					cookie = xstrdup(skip_whitespace(iobuf + sizeof("Cookie:")-1));
-			} else if (STRNCASECMP(iobuf, "Content-Type:") == 0) {
-				if (!content_type)
-					content_type = xstrdup(skip_whitespace(iobuf + sizeof("Content-Type:")-1));
-			} else if (STRNCASECMP(iobuf, "Referer:") == 0) {
-				if (!G.referer)
-					G.referer = xstrdup(skip_whitespace(iobuf + sizeof("Referer:")-1));
-			} else if (STRNCASECMP(iobuf, "User-Agent:") == 0) {
-				if (!G.user_agent)
-					G.user_agent = xstrdup(skip_whitespace(iobuf + sizeof("User-Agent:")-1));
-			} else if (STRNCASECMP(iobuf, "Host:") == 0) {
-				if (!G.host)
-					G.host = xstrdup(skip_whitespace(iobuf + sizeof("Host:")-1));
-			} else if (STRNCASECMP(iobuf, "Accept:") == 0) {
-				if (!G.http_accept)
-					G.http_accept = xstrdup(skip_whitespace(iobuf + sizeof("Accept:")-1));
-			} else if (STRNCASECMP(iobuf, "Accept-Language:") == 0) {
-				if (!G.http_accept_language)
-					G.http_accept_language = xstrdup(skip_whitespace(iobuf + sizeof("Accept-Language:")-1));
-			}
-#endif
-#if ENABLE_FEATURE_HTTPD_BASIC_AUTH
-			if (STRNCASECMP(iobuf, "Authorization:") == 0) {
-				/* We only allow Basic credentials.
-				 * It shows up as "Authorization: Basic <user>:<passwd>" where
-				 * "<user>:<passwd>" is base64 encoded.
-				 */
-				tptr = skip_whitespace(iobuf + sizeof("Authorization:")-1);
-				if (STRNCASECMP(tptr, "Basic") != 0)
-					continue;
-				tptr += sizeof("Basic")-1;
-				/* decodeBase64() skips whitespace itself */
-				decodeBase64(tptr);
-				authorized = check_user_passwd(urlcopy, tptr);
-			}
-#endif
-#if ENABLE_FEATURE_HTTPD_RANGES
-			if (STRNCASECMP(iobuf, "Range:") == 0) {
-				/* We know only bytes=NNN-[MMM] */
-				char *s = skip_whitespace(iobuf + sizeof("Range:")-1);
-				if (is_prefixed_with(s, "bytes=")) {
-					s += sizeof("bytes=")-1;
-					range_start = BB_STRTOOFF(s, &s, 10);
-					if (s[0] != '-' || range_start < 0) {
-						range_start = -1;
-					} else if (s[1]) {
-						range_end = BB_STRTOOFF(s+1, NULL, 10);
-						if (errno || range_end < range_start)
-							range_start = -1;
-					}
-				}
-			}
-#endif
-#if ENABLE_FEATURE_HTTPD_GZIP
-			if (STRNCASECMP(iobuf, "Accept-Encoding:") == 0) {
-				/* Note: we do not support "gzip;q=0"
-				 * method of _disabling_ gzip
-				 * delivery. No one uses that, though */
-				const char *s = strstr(iobuf, "gzip");
-				if (s) {
-					// want more thorough checks?
-					//if (s[-1] == ' '
-					// || s[-1] == ','
-					// || s[-1] == ':'
-					//) {
-						content_gzip = 1;
-					//}
-				}
-			}
-#endif
-		} /* while extra header reading */
-	}
-
-	/* We are done reading headers, disable peer timeout */
-	alarm(0);
-
-	if (strcmp(bb_basename(urlcopy), HTTPD_CONF) == 0 || !ip_allowed) {
-		/* protect listing [/path]/httpd.conf or IP deny */
-		send_headers_and_exit(HTTP_FORBIDDEN);
-	}
-
-#if ENABLE_FEATURE_HTTPD_BASIC_AUTH
-	/* Case: no "Authorization:" was seen, but page might require passwd.
-	 * Check that with dummy user:pass */
-	if (authorized < 0)
-		authorized = check_user_passwd(urlcopy, (char *) "");
-	if (!authorized)
-		send_headers_and_exit(HTTP_UNAUTHORIZED);
-#endif
-
-	if (found_moved_temporarily) {
-		send_headers_and_exit(HTTP_MOVED_TEMPORARILY);
-	}
-
-#if ENABLE_FEATURE_HTTPD_PROXY
-	if (proxy_entry != NULL) {
-		int proxy_fd;
-		len_and_sockaddr *lsa;
-
-		lsa = host2sockaddr(proxy_entry->host_port, 80);
-		if (lsa == NULL)
-			send_headers_and_exit(HTTP_INTERNAL_SERVER_ERROR);
-		proxy_fd = socket(lsa->u.sa.sa_family, SOCK_STREAM, 0);
-		if (proxy_fd < 0)
-			send_headers_and_exit(HTTP_INTERNAL_SERVER_ERROR);
-		if (connect(proxy_fd, &lsa->u.sa, lsa->len) < 0)
-			send_headers_and_exit(HTTP_INTERNAL_SERVER_ERROR);
-		fdprintf(proxy_fd, "%s %s%s%s%s HTTP/%c.%c\r\n",
-				prequest, /* GET or POST */
-				proxy_entry->url_to, /* url part 1 */
-				urlcopy + strlen(proxy_entry->url_from), /* url part 2 */
-				(g_query ? "?" : ""), /* "?" (maybe) */
-				(g_query ? g_query : ""), /* query string (maybe) */
-				http_major_version, http_minor_version);
-		header_ptr[0] = '\r';
-		header_ptr[1] = '\n';
-		header_ptr += 2;
-		write(proxy_fd, header_buf, header_ptr - header_buf);
-		free(header_buf); /* on the order of 8k, free it */
-		cgi_io_loop_and_exit(proxy_fd, proxy_fd, length);
-	}
-#endif
-
 	tptr = urlcopy + 1;      /* skip first '/' */
 
 #if ENABLE_FEATURE_HTTPD_CGI
@@ -2435,7 +2314,7 @@ static void handle_incoming_and_exit(const len_and_sockaddr *fromAddr)
 			/* protect listing "cgi-bin/" */
 			send_headers_and_exit(HTTP_FORBIDDEN);
 		}
-		send_cgi_and_exit(urlcopy, urlcopy, prequest, length, cookie, content_type);
+		cgi_type = CGI_NORMAL;
 	}
 #endif
 
@@ -2456,25 +2335,186 @@ static void handle_incoming_and_exit(const len_and_sockaddr *fromAddr)
 			Htaccess *cur;
 			for (cur = script_i; cur; cur = cur->next) {
 				if (strcmp(cur->before_colon + 1, suffix) == 0) {
-					send_cgi_and_exit(urlcopy, urlcopy, prequest, length, cookie, content_type);
+					cgi_type = CGI_INTERPRETER;
+					break;
 				}
 			}
 		}
 #endif
-		file_size = sb.st_size;
-		last_mod = sb.st_mtime;
+		if (!found_moved_temporarily) {
+			file_size = sb.st_size;
+			last_mod = sb.st_mtime;
+		}
 	}
 #if ENABLE_FEATURE_HTTPD_CGI
 	else if (urlp[-1] == '/') {
 		/* It's a dir URL and there is no index.html
 		 * Try cgi-bin/index.cgi */
 		if (access("/cgi-bin/index.cgi"+1, X_OK) == 0) {
-			urlp[0] = '\0'; /* remove index_page */
-			send_cgi_and_exit("/cgi-bin/index.cgi", urlcopy, prequest, length, cookie, content_type);
+			cgi_type = CGI_INDEX;
 		}
 	}
-	/* else fall through to send_file, it errors out if open fails: */
+#endif
+	urlp[0] = '\0';
 
+#if ENABLE_FEATURE_HTTPD_CGI
+	total_headers_len = 0;
+#endif
+
+	/* Read until blank line */
+	while (1) {
+		unsigned iobuf_len = get_line();
+		if (!iobuf_len)
+			break; /* EOF or error or empty line */
+#if ENABLE_FEATURE_HTTPD_CGI
+		/* Prevent unlimited growth of HTTP_xyz envvars */
+		total_headers_len += iobuf_len;
+		if (total_headers_len >= MAX_HTTP_HEADERS_SIZE)
+			send_headers_and_exit(HTTP_ENTITY_TOO_LARGE);
+#endif
+		if (DEBUG)
+			bb_error_msg("header: '%s'", iobuf);
+#if ENABLE_FEATURE_HTTPD_CGI || ENABLE_FEATURE_HTTPD_PROXY
+		/* Try and do our best to parse more lines */
+		if (STRNCASECMP(iobuf, "Content-Length:") == 0) {
+			/* extra read only for POST */
+			if (prequest != request_GET
+# if ENABLE_FEATURE_HTTPD_CGI
+			 && prequest != request_HEAD
+# endif
+			) {
+				tptr = skip_whitespace(iobuf + sizeof("Content-Length:") - 1);
+				if (!tptr[0])
+					send_headers_and_exit(HTTP_BAD_REQUEST);
+				/* not using strtoul: it ignores leading minus! */
+				length = bb_strtou(tptr, NULL, 10);
+				/* length is "ulong", but we need to pass it to int later */
+				if (errno || length > INT_MAX)
+					send_headers_and_exit(HTTP_BAD_REQUEST);
+			}
+			continue;
+		}
+#endif
+#if ENABLE_FEATURE_HTTPD_BASIC_AUTH
+		if (STRNCASECMP(iobuf, "Authorization:") == 0) {
+			/* We only allow Basic credentials.
+			 * It shows up as "Authorization: Basic <user>:<passwd>" where
+			 * "<user>:<passwd>" is base64 encoded.
+			 */
+			tptr = skip_whitespace(iobuf + sizeof("Authorization:")-1);
+			if (STRNCASECMP(tptr, "Basic") == 0) {
+				tptr += sizeof("Basic")-1;
+				/* decodeBase64() skips whitespace itself */
+				decodeBase64(tptr);
+				authorized = check_user_passwd(urlcopy, tptr);
+				continue;
+			}
+		}
+#endif
+#if ENABLE_FEATURE_HTTPD_RANGES
+		if (STRNCASECMP(iobuf, "Range:") == 0) {
+			/* We know only bytes=NNN-[MMM] */
+			char *s = skip_whitespace(iobuf + sizeof("Range:")-1);
+			if (is_prefixed_with(s, "bytes=")) {
+				s += sizeof("bytes=")-1;
+				range_start = BB_STRTOOFF(s, &s, 10);
+				if (s[0] != '-' || range_start < 0) {
+					range_start = -1;
+				} else if (s[1]) {
+					range_end = BB_STRTOOFF(s+1, NULL, 10);
+					if (errno || range_end < range_start)
+						range_start = -1;
+				}
+			}
+			continue;
+		}
+#endif
+#if ENABLE_FEATURE_HTTPD_GZIP
+		if (STRNCASECMP(iobuf, "Accept-Encoding:") == 0) {
+			/* Note: we do not support "gzip;q=0"
+			 * method of _disabling_ gzip
+			 * delivery. No one uses that, though */
+			const char *s = strstr(iobuf, "gzip");
+			if (s) {
+				// want more thorough checks?
+				//if (s[-1] == ' '
+				// || s[-1] == ','
+				// || s[-1] == ':'
+				//) {
+					content_gzip = 1;
+				//}
+			}
+			continue;
+		}
+#endif
+#if ENABLE_FEATURE_HTTPD_CGI
+		if (cgi_type != CGI_NONE) {
+			bool ct = (STRNCASECMP(iobuf, "Content-Type:") == 0);
+			char *cp;
+			char *colon = strchr(iobuf, ':');
+
+			if (!colon)
+				continue;
+			cp = iobuf;
+			while (cp < colon) {
+				/* a-z => A-Z, not-alnum => _ */
+				char c = (*cp & ~0x20); /* toupper for A-Za-z, undef for others */
+				if ((unsigned)(c - 'A') <= ('Z' - 'A')) {
+					*cp++ = c;
+					continue;
+				}
+				if (!isdigit(*cp))
+					*cp = '_';
+				cp++;
+			}
+			/* "Content-Type:" gets no HTTP_ prefix, all others do */
+			cp = xasprintf(ct ? "HTTP_%.*s=%s" + 5 : "HTTP_%.*s=%s",
+				(int)(colon - iobuf), iobuf,
+				skip_whitespace(colon + 1)
+			);
+			putenv(cp);
+		}
+#endif
+	} /* while extra header reading */
+
+	/* We are done reading headers, disable peer timeout */
+	alarm(0);
+
+	if (strcmp(bb_basename(urlcopy), HTTPD_CONF) == 0) {
+		/* protect listing [/path]/httpd.conf or IP deny */
+		send_headers_and_exit(HTTP_FORBIDDEN);
+	}
+
+#if ENABLE_FEATURE_HTTPD_BASIC_AUTH
+	/* Case: no "Authorization:" was seen, but page might require passwd.
+	 * Check that with dummy user:pass */
+	if (authorized < 0)
+		authorized = check_user_passwd(urlcopy, (char *) "");
+	if (!authorized)
+		send_headers_and_exit(HTTP_UNAUTHORIZED);
+#endif
+
+	if (found_moved_temporarily) {
+		send_headers_and_exit(HTTP_MOVED_TEMPORARILY);
+	}
+
+	tptr = urlcopy + 1;      /* skip first '/' */
+
+#if ENABLE_FEATURE_HTTPD_CGI
+	if (cgi_type != CGI_NONE) {
+		send_cgi_and_exit(
+			(cgi_type == CGI_INDEX) ? "/cgi-bin/index.cgi"
+			/*CGI_NORMAL or CGI_INTERPRETER*/ : urlcopy,
+			urlcopy, prequest, length
+		);
+	}
+#endif
+
+	if (urlp[-1] == '/') {
+		strcpy(urlp, index_page);
+	}
+
+#if ENABLE_FEATURE_HTTPD_CGI
 	if (prequest != request_GET && prequest != request_HEAD) {
 		/* POST for files does not make sense */
 		send_headers_and_exit(HTTP_NOT_IMPLEMENTED);
@@ -2593,7 +2633,9 @@ static void mini_httpd_inetd(void)
 
 static void sighup_handler(int sig UNUSED_PARAM)
 {
+	int sv = errno;
 	parse_conf(DEFAULT_PATH_HTTPD_CONF, SIGNALED_PARSE);
+	errno = sv;
 }
 
 enum {
@@ -2692,10 +2734,15 @@ int httpd_main(int argc UNUSED_PARAM, char **argv)
 #if !BB_MMU
 	if (!(opt & OPT_FOREGROUND)) {
 		bb_daemonize_or_rexec(0, argv); /* don't change current directory */
+		re_execed = 0; /* for the following chdir to work */
 	}
 #endif
+	/* Chdir to home (unless we were re_exec()ed for NOMMU case
+	 * in mini_httpd_nommu(): we are already in the home dir then).
+	 */
+	if (!re_execed)
+		xchdir(home_httpd);
 
-	xchdir(home_httpd);
 	if (!(opt & OPT_INETD)) {
 		signal(SIGCHLD, SIG_IGN);
 		server_socket = openServer();
@@ -2704,7 +2751,7 @@ int httpd_main(int argc UNUSED_PARAM, char **argv)
 		if (opt & OPT_SETUID) {
 			if (ugid.gid != (gid_t)-1) {
 				if (setgroups(1, &ugid.gid) == -1)
-					bb_perror_msg_and_die("setgroups");
+					bb_simple_perror_msg_and_die("setgroups");
 				xsetgid(ugid.gid);
 			}
 			xsetuid(ugid.uid);
