@@ -8,7 +8,7 @@
  * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 //config:config KILL
-//config:	bool "kill (2.6 kb)"
+//config:	bool "kill (3.1 kb)"
 //config:	default y
 //config:	help
 //config:	The command kill sends the specified signal to the specified
@@ -268,7 +268,7 @@ int kill_main(int argc UNUSED_PARAM, char **argv)
 #if ENABLE_KILL || ENABLE_KILLALL
 	/* Pid or name is required for kill/killall */
 	if (!arg) {
-		bb_error_msg("you need to specify whom to kill");
+		bb_simple_error_msg("you need to specify whom to kill");
 		return EXIT_FAILURE;
 	}
 
