@@ -9,26 +9,26 @@
  * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 //config:config LOGREAD
-//config:	bool "logread"
+//config:	bool "logread (4.8 kb)"
 //config:	default y
 //WRONG: it should be compilable without SYSLOG=y:
 //WRONG:	depends on FEATURE_IPC_SYSLOG
 //config:	help
-//config:	  If you enabled Circular Buffer support, you almost
-//config:	  certainly want to enable this feature as well. This
-//config:	  utility will allow you to read the messages that are
-//config:	  stored in the syslogd circular buffer.
+//config:	If you enabled Circular Buffer support, you almost
+//config:	certainly want to enable this feature as well. This
+//config:	utility will allow you to read the messages that are
+//config:	stored in the syslogd circular buffer.
 //config:
 //config:config FEATURE_LOGREAD_REDUCED_LOCKING
 //config:	bool "Double buffering"
 //config:	default y
 //config:	depends on LOGREAD
 //config:	help
-//config:	  'logread' output to slow serial terminals can have
-//config:	  side effects on syslog because of the semaphore.
-//config:	  This option make logread to double buffer copy
-//config:	  from circular buffer, minimizing semaphore
-//config:	  contention at some minor memory expense.
+//config:	'logread' output to slow serial terminals can have
+//config:	side effects on syslog because of the semaphore.
+//config:	This option make logread to double buffer copy
+//config:	from circular buffer, minimizing semaphore
+//config:	contention at some minor memory expense.
 //config:
 
 //applet:IF_LOGREAD(APPLET(logread, BB_DIR_SBIN, BB_SUID_DROP))
@@ -88,7 +88,7 @@ static void error_exit(const char *str)
 }
 #else
 /* On Linux, shmdt is not mandatory on exit */
-# define error_exit(str) bb_perror_msg_and_die(str)
+# define error_exit(str) bb_simple_perror_msg_and_die(str)
 #endif
 
 /*
