@@ -7,7 +7,7 @@
  * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 //config:config LOSETUP
-//config:	bool "losetup (5.4 kb)"
+//config:	bool "losetup (5.5 kb)"
 //config:	default y
 //config:	select PLATFORM_LINUX
 //config:	help
@@ -20,12 +20,14 @@
 //kbuild:lib-$(CONFIG_LOSETUP) += losetup.o
 
 //usage:#define losetup_trivial_usage
-//usage:       "[-r] [-o OFS] {-f|LOOPDEV} FILE - associate loop devices\n"
-//usage:       "	losetup -d LOOPDEV - disassociate\n"
-//usage:       "	losetup -a - show status\n"
-//usage:       "	losetup -f - show next free loop device"
+//usage:       "[-rP] [-o OFS] {-f|LOOPDEV} FILE: associate loop devices\n"
+//usage:       "	losetup -c LOOPDEV: reread file size\n"
+//usage:       "	losetup -d LOOPDEV: disassociate\n"
+//usage:       "	losetup -a: show status\n"
+//usage:       "	losetup -f: show next free loop device"
 //usage:#define losetup_full_usage "\n\n"
 //usage:       "	-o OFS	Start OFS bytes into FILE"
+//usage:     "\n	-P	Scan for partitions"
 //usage:     "\n	-r	Read-only"
 //usage:     "\n	-f	Show/use next free loop device"
 //usage:
@@ -34,8 +36,9 @@
 //usage:       "(if any), or disassociate it (with -d). The display shows the offset\n"
 //usage:       "and filename of the file the loop device is currently bound to.\n\n"
 //usage:       "Two arguments (losetup /dev/loop1 file.img) create a new association,\n"
-//usage:       "with an optional offset (-o 12345). Encryption is not yet supported.\n"
-//usage:       "losetup -f will show the first loop free loop device\n\n"
+//usage:       "with optional partition scanning (creates /dev/loop1p1, /dev/loop1p2\n"
+//usage:       "etc. with -P) and with an optional offset (-o 12345). Encryption is\n"
+//usage:       "not yet supported. losetup -f will show the first free loop device\n\n"
 
 #include "libbb.h"
 
@@ -50,14 +53,16 @@ int losetup_main(int argc UNUSED_PARAM, char **argv)
 	char *opt_o;
 	char dev[LOOP_NAMESIZE];
 	enum {
-		OPT_d = (1 << 0),
-		OPT_o = (1 << 1),
-		OPT_f = (1 << 2),
-		OPT_a = (1 << 3),
-		OPT_r = (1 << 4), /* must be last */
+		OPT_c = (1 << 0),
+		OPT_d = (1 << 1),
+		OPT_P = (1 << 2),
+		OPT_o = (1 << 3),
+		OPT_f = (1 << 4),
+		OPT_a = (1 << 5),
+		OPT_r = (1 << 6),
 	};
 
-	opt = getopt32(argv, "^" "do:far" "\0" "?2:d--ofar:a--ofr", &opt_o);
+	opt = getopt32(argv, "^" "cdPo:far" "\0" "?2:d--Pofar:a--Pofr", &opt_o);
 	argv += optind;
 
 	/* LOOPDEV */
@@ -73,6 +78,16 @@ int losetup_main(int argc UNUSED_PARAM, char **argv)
 		return EXIT_SUCCESS;
 	}
 
+	/* -c LOOPDEV */
+	if (opt == OPT_c && argv[0]) {
+		int fd = xopen(argv[0], O_RDONLY);
+#ifndef LOOP_SET_CAPACITY
+# define LOOP_SET_CAPACITY 0x4C07
+#endif
+		xioctl(fd, LOOP_SET_CAPACITY, /*ignored:*/0);
+		return EXIT_SUCCESS;
+	}
+
 	/* -d LOOPDEV */
 	if (opt == OPT_d && argv[0]) {
 		if (del_loop(argv[0]))
@@ -99,11 +114,17 @@ int losetup_main(int argc UNUSED_PARAM, char **argv)
 	/* contains -f */
 	if (opt & OPT_f) {
 		char *s;
-		int n = 0;
+		int n;
 
+		n = get_free_loop();
+		if (n == -1)
+			bb_simple_error_msg_and_die("no free loop devices");
+		if (n < 0) /* n == -2: no /dev/loop-control, use legacy method */
+			n = 0;
+		/* or: n >= 0: the number of next free loopdev, just verify it */
 		do {
 			if (n > MAX_LOOP_NUM)
-				bb_error_msg_and_die("no free loop devices");
+				bb_simple_error_msg_and_die("no free loop devices");
 			sprintf(dev, LOOP_FORMAT, n++);
 			s = query_loop(dev);
 			free(s);
@@ -115,7 +136,7 @@ int losetup_main(int argc UNUSED_PARAM, char **argv)
 		}
 	}
 
-	/* [-r] [-o OFS] {-f|LOOPDEV} FILE */
+	/* [-rP] [-o OFS] {-f|LOOPDEV} FILE */
 	if (argv[0] && ((opt & OPT_f) || argv[1])) {
 		unsigned long long offset = 0;
 		char *d = dev;
@@ -126,7 +147,11 @@ int losetup_main(int argc UNUSED_PARAM, char **argv)
 			d = *argv++;
 
 		if (argv[0]) {
-			if (set_loop(&d, argv[0], offset, (opt & OPT_r) ? BB_LO_FLAGS_READ_ONLY : 0) < 0)
+			unsigned flags = (opt & OPT_r) ? BB_LO_FLAGS_READ_ONLY : 0;
+			if (opt & OPT_P) {
+				flags |= BB_LO_FLAGS_PARTSCAN;
+			}
+			if (set_loop(&d, argv[0], offset, flags) < 0)
 				bb_simple_perror_msg_and_die(argv[0]);
 			return EXIT_SUCCESS;
 		}
