@@ -64,15 +64,30 @@
 //config:	These devices will request userspace look up the files in
 //config:	/lib/firmware/ and if it exists, send it to the kernel for
 //config:	loading into the hardware.
+//config:
+//config:config FEATURE_MDEV_DAEMON
+//config:	bool "Support daemon mode"
+//config:	default y
+//config:	depends on MDEV
+//config:	help
+//config:	Adds the -d option to run mdev in daemon mode handling hotplug
+//config:	events from the kernel like udev. If the system generates many
+//config:	hotplug events this mode of operation will consume less
+//config:	resources than registering mdev as hotplug helper or using the
+//config:	uevent applet.
 
 //applet:IF_MDEV(APPLET(mdev, BB_DIR_SBIN, BB_SUID_DROP))
 
 //kbuild:lib-$(CONFIG_MDEV) += mdev.o
 
 //usage:#define mdev_trivial_usage
-//usage:       "[-s]"
+//usage:       "[-s]" IF_FEATURE_MDEV_DAEMON(" | [-df]")
 //usage:#define mdev_full_usage "\n\n"
 //usage:       "mdev -s is to be run during boot to scan /sys and populate /dev.\n"
+//usage:	IF_FEATURE_MDEV_DAEMON(
+//usage:       "mdev -d[f]: daemon, listen on netlink.\n"
+//usage:       "	-f: stay in foreground.\n"
+//usage:	)
 //usage:       "\n"
 //usage:       "Bare mdev is a kernel hotplug helper. To activate it:\n"
 //usage:       "	echo /sbin/mdev >/proc/sys/kernel/hotplug\n"
@@ -98,6 +113,7 @@
 #include "libbb.h"
 #include "common_bufsiz.h"
 #include "xregex.h"
+#include <linux/netlink.h>
 
 /* "mdev -s" scans /sys/class/xxx, looking for directories which have dev
  * file (it is of the form "M:m\n"). Example: /sys/class/tty/tty0/dev
@@ -234,21 +250,30 @@
 
 #if DEBUG_LVL >= 1
 # define dbg1(...) do { if (G.verbose) bb_error_msg(__VA_ARGS__); } while(0)
+# define dbg1s(msg) do { if (G.verbose) bb_simple_error_msg(msg); } while(0)
 #else
 # define dbg1(...) ((void)0)
+# define dbg1s(msg) ((void)0)
 #endif
 #if DEBUG_LVL >= 2
 # define dbg2(...) do { if (G.verbose >= 2) bb_error_msg(__VA_ARGS__); } while(0)
+# define dbg2s(msg) do { if (G.verbose >= 2) bb_simple_error_msg(msg); } while(0)
 #else
 # define dbg2(...) ((void)0)
+# define dbg2s(msg) ((void)0)
 #endif
 #if DEBUG_LVL >= 3
 # define dbg3(...) do { if (G.verbose >= 3) bb_error_msg(__VA_ARGS__); } while(0)
+# define dbg3s(msg) do { if (G.verbose >= 3) bb_simple_error_msg(msg); } while(0)
 #else
 # define dbg3(...) ((void)0)
+# define dbg3s(msg) ((void)0)
 #endif
 
 
+#ifndef SO_RCVBUFFORCE
+#define SO_RCVBUFFORCE 33
+#endif
 static const char keywords[] ALIGN1 = "add\0remove\0"; // "change\0"
 enum { OP_add, OP_remove };
 
@@ -1002,7 +1027,7 @@ wait_for_seqfile(unsigned expected_seq)
 			/* seed file: write out seq ASAP */
 			xwrite_str(seq_fd, utoa(expected_seq));
 			xlseek(seq_fd, 0, SEEK_SET);
-			dbg2("first seq written");
+			dbg2s("first seq written");
 			break;
 		}
 		seqbufnum = atoll(seqbuf);
@@ -1047,17 +1072,141 @@ static void signal_mdevs(unsigned my_pid)
 	}
 }
 
+static void process_action(char *temp, unsigned my_pid)
+{
+	char *fw;
+	char *seq;
+	char *action;
+	char *env_devname;
+	char *env_devpath;
+	unsigned seqnum = seqnum; /* for compiler */
+	int seq_fd;
+	smalluint op;
+
+	/* Hotplug:
+	 * env ACTION=... DEVPATH=... SUBSYSTEM=... [SEQNUM=...] mdev
+	 * ACTION can be "add", "remove", "change"
+	 * DEVPATH is like "/block/sda" or "/class/input/mice"
+	 */
+	env_devname = getenv("DEVNAME"); /* can be NULL */
+	G.subsystem = getenv("SUBSYSTEM");
+	action = getenv("ACTION");
+	env_devpath = getenv("DEVPATH");
+	if (!action || !env_devpath /*|| !G.subsystem*/)
+		bb_show_usage();
+	fw = getenv("FIRMWARE");
+	seq = getenv("SEQNUM");
+	op = index_in_strings(keywords, action);
+
+	if (my_pid)
+		open_mdev_log(seq, my_pid);
+
+	seq_fd = -1;
+	if (my_pid && seq) {
+		seqnum = atoll(seq);
+		seq_fd = wait_for_seqfile(seqnum);
+	}
+
+	dbg1("%s "
+		"ACTION:%s SEQNUM:%s SUBSYSTEM:%s DEVNAME:%s DEVPATH:%s"
+		"%s%s",
+		curtime(),
+		action, seq, G.subsystem, env_devname, env_devpath,
+		fw ? " FW:" : "", fw ? fw : ""
+	);
+
+	snprintf(temp, PATH_MAX, "/sys%s", env_devpath);
+	if (op == OP_remove) {
+		/* Ignoring "remove firmware". It was reported
+		 * to happen and to cause erroneous deletion
+		 * of device nodes. */
+		if (!fw)
+			make_device(env_devname, temp, op);
+	}
+	else {
+		make_device(env_devname, temp, op);
+		if (ENABLE_FEATURE_MDEV_LOAD_FIRMWARE) {
+			if (op == OP_add && fw)
+				load_firmware(fw, temp);
+		}
+	}
+
+	if (seq_fd >= 0) {
+		xwrite_str(seq_fd, utoa(seqnum + 1));
+		signal_mdevs(my_pid);
+	}
+}
+
+static void initial_scan(char *temp)
+{
+	struct stat st;
+
+	xstat("/", &st);
+	G.root_major = major(st.st_dev);
+	G.root_minor = minor(st.st_dev);
+
+	putenv((char*)"ACTION=add");
+
+	/* Create all devices from /sys/dev hierarchy */
+	recursive_action("/sys/dev",
+			 ACTION_RECURSE | ACTION_FOLLOWLINKS,
+			 fileAction, dirAction, temp, 0);
+}
+
+#if ENABLE_FEATURE_MDEV_DAEMON
+
+/* uevent applet uses 16k buffer, and mmaps it before every read */
+# define BUFFER_SIZE (2 * 1024)
+# define RCVBUF (2 * 1024 * 1024)
+# define MAX_ENV 32
+
+static void daemon_loop(char *temp, int fd)
+{
+	for (;;) {
+		char netbuf[BUFFER_SIZE];
+		char *env[MAX_ENV];
+		char *s, *end;
+		ssize_t len;
+		int idx;
+
+		len = safe_read(fd, netbuf, sizeof(netbuf) - 1);
+		if (len < 0) {
+			bb_simple_perror_msg_and_die("read");
+		}
+		end = netbuf + len;
+		*end = '\0';
+
+		idx = 0;
+		s = netbuf;
+		while (s < end && idx < MAX_ENV) {
+			if (endofname(s)[0] == '=') {
+				env[idx++] = s;
+				putenv(s);
+			}
+			s += strlen(s) + 1;
+		}
+
+		process_action(temp, 0);
+
+		while (idx)
+			bb_unsetenv(env[--idx]);
+	}
+}
+#endif
+
 int mdev_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int mdev_main(int argc UNUSED_PARAM, char **argv)
 {
+	enum {
+		MDEV_OPT_SCAN       = 1 << 0,
+		MDEV_OPT_DAEMON     = 1 << 1,
+		MDEV_OPT_FOREGROUND = 1 << 2,
+	};
+	int opt;
 	RESERVE_CONFIG_BUFFER(temp, PATH_MAX + SCRATCH_SIZE);
 
 	INIT_G();
 
-#if ENABLE_FEATURE_MDEV_CONF
-	G.filename = "/etc/mdev.conf";
-#endif
-
 	/* We can be called as hotplug helper */
 	/* Kernel cannot provide suitable stdio fds for us, do it ourself */
 	bb_sanitize_stdio();
@@ -1067,90 +1216,55 @@ int mdev_main(int argc UNUSED_PARAM, char **argv)
 
 	xchdir("/dev");
 
-	if (argv[1] && strcmp(argv[1], "-s") == 0) {
+	opt = getopt32(argv, "s" IF_FEATURE_MDEV_DAEMON("df"));
+
+#if ENABLE_FEATURE_MDEV_CONF
+	G.filename = "/etc/mdev.conf";
+	if (opt & (MDEV_OPT_SCAN|MDEV_OPT_DAEMON)) {
+		/* Same as xrealloc_vector(NULL, 4, 0): */
+		G.rule_vec = xzalloc((1 << 4) * sizeof(*G.rule_vec));
+	}
+#endif
+
+#if ENABLE_FEATURE_MDEV_DAEMON
+	if (opt & MDEV_OPT_DAEMON) {
+		/*
+		 * Daemon mode listening on uevent netlink socket.
+		 */
+		int fd;
+
+		/* Subscribe for UEVENT kernel messages */
+		/* Without a sufficiently big RCVBUF, a ton of simultaneous events
+		 * can trigger ENOBUFS on read, which is unrecoverable.
+		 * Reproducer:
+		 *	mdev -d
+		 *	find /sys -name uevent -exec sh -c 'echo add >"{}"' ';'
+		 */
+		fd = create_and_bind_to_netlink(NETLINK_KOBJECT_UEVENT, 1 << 0, RCVBUF);
+
+		/*
+		 * Make inital scan after the uevent socket is alive and
+		 * _before_ we fork away.
+		 */
+		initial_scan(temp);
+
+		if (!(opt & MDEV_OPT_FOREGROUND))
+			bb_daemonize_or_rexec(0, argv);
+
+		open_mdev_log(NULL, getpid());
+
+		daemon_loop(temp, fd);
+	}
+#endif
+	if (opt & MDEV_OPT_SCAN) {
 		/*
 		 * Scan: mdev -s
 		 */
-		struct stat st;
-
-#if ENABLE_FEATURE_MDEV_CONF
-		/* Same as xrealloc_vector(NULL, 4, 0): */
-		G.rule_vec = xzalloc((1 << 4) * sizeof(*G.rule_vec));
-#endif
-		xstat("/", &st);
-		G.root_major = major(st.st_dev);
-		G.root_minor = minor(st.st_dev);
-
-		putenv((char*)"ACTION=add");
-
-		/* Create all devices from /sys/dev hierarchy */
-		recursive_action("/sys/dev",
-				 ACTION_RECURSE | ACTION_FOLLOWLINKS,
-				 fileAction, dirAction, temp, 0);
+		initial_scan(temp);
 	} else {
-		char *fw;
-		char *seq;
-		char *action;
-		char *env_devname;
-		char *env_devpath;
-		unsigned my_pid;
-		unsigned seqnum = seqnum; /* for compiler */
-		int seq_fd;
-		smalluint op;
-
-		/* Hotplug:
-		 * env ACTION=... DEVPATH=... SUBSYSTEM=... [SEQNUM=...] mdev
-		 * ACTION can be "add", "remove", "change"
-		 * DEVPATH is like "/block/sda" or "/class/input/mice"
-		 */
-		env_devname = getenv("DEVNAME"); /* can be NULL */
-		G.subsystem = getenv("SUBSYSTEM");
-		action = getenv("ACTION");
-		env_devpath = getenv("DEVPATH");
-		if (!action || !env_devpath /*|| !G.subsystem*/)
-			bb_show_usage();
-		fw = getenv("FIRMWARE");
-		seq = getenv("SEQNUM");
-		op = index_in_strings(keywords, action);
-
-		my_pid = getpid();
-		open_mdev_log(seq, my_pid);
-
-		seq_fd = -1;
-		if (seq) {
-			seqnum = atoll(seq);
-			seq_fd = wait_for_seqfile(seqnum);
-		}
-
-		dbg1("%s "
-			"ACTION:%s SUBSYSTEM:%s DEVNAME:%s DEVPATH:%s"
-			"%s%s",
-			curtime(),
-			action, G.subsystem, env_devname, env_devpath,
-			fw ? " FW:" : "", fw ? fw : ""
-		);
-
-		snprintf(temp, PATH_MAX, "/sys%s", env_devpath);
-		if (op == OP_remove) {
-			/* Ignoring "remove firmware". It was reported
-			 * to happen and to cause erroneous deletion
-			 * of device nodes. */
-			if (!fw)
-				make_device(env_devname, temp, op);
-		}
-		else {
-			make_device(env_devname, temp, op);
-			if (ENABLE_FEATURE_MDEV_LOAD_FIRMWARE) {
-				if (op == OP_add && fw)
-					load_firmware(fw, temp);
-			}
-		}
+		process_action(temp, getpid());
 
 		dbg1("%s exiting", curtime());
-		if (seq_fd >= 0) {
-			xwrite_str(seq_fd, utoa(seqnum + 1));
-			signal_mdevs(my_pid);
-		}
 	}
 
 	if (ENABLE_FEATURE_CLEAN_UP)
